package BBUVA;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.persistence.CascadeType;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.ManyToMany;
import javax.persistence.Transient;

@Entity
public class Producto implements Serializable {

    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    private Long id;
    private String numeroProducto;
    @ManyToMany(cascade = CascadeType.ALL)
    private List<Titular> mTitular;
    @Transient
    private List<Movimiento> mMovimiento;

    public Producto () {
        mMovimiento = new ArrayList<Movimiento>();
        mTitular = new ArrayList<Titular>();
    }

    public void addTitular(Titular get) {
        mTitular.add(get);
    }

    public boolean existeTitular(String identificacion) {
        for(int i=0;i<this.mTitular.size();i++)
            if(this.mTitular.get(i).getDocumentacion().compareTo(identificacion)==0)
                return true;
        return false;
    }

    public String getNumeroProducto () {
        return numeroProducto;
    }

    public void setNumeroProducto (String val) {
        this.numeroProducto = val;
    }

    public List<Titular> getTitular () {
        return mTitular;
    }

    public void setTitular (List val) {
        this.mTitular = val;
    }

    public List<Movimiento> getMovimiento () {
        return mMovimiento;
    }

    public void setMovimiento (List<Movimiento> val) {
        mMovimiento = val;
    }

    public float getSaldo () {
        float saldo=0;
        for(int i=0;i<mMovimiento.size();i++)
            saldo = saldo + mMovimiento.get(i).getCambioSaldo();
        return saldo;
    }

    public List<Movimiento> getUltimosMovimientos() {
        List<Movimiento> devolver=new ArrayList<Movimiento>();
        int movs=mMovimiento.size();
        if(movs>5) movs=5;
        for(int i=mMovimiento.size()-movs;i<mMovimiento.size();i++) {
            devolver.add(mMovimiento.get(i));
        }
        return devolver;
    }

    public List<Movimiento> getOperRemesas() {
        List<Movimiento> devolver=new ArrayList<Movimiento>();
        for(int i=0;i<mMovimiento.size();i++) {
            if(mMovimiento.get(i).getClass()==OperRemesa.class)
                devolver.add(mMovimiento.get(i));
        }
        return devolver;
    }

    @Override
    public String toString() {
        return this.getNumeroProducto()+"\t"+this.getSaldo();
    }

    public String descripcion() {
        String cadena=this.toString();
        cadena+="\nTITULARES";
        for(int i=0;i<this.getTitular().size();i++) {
            cadena+="\n   "+ this.getTitular().get(i)+"\n";
        }
        cadena+="\n";
        return cadena;
    }

    public Long getId() {
        return id;
    }

    public void setId(Long id) {
        this.id = id;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (id != null ? id.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the documentacion fields are not set
        if (!(object instanceof Producto)) {
            return false;
        }
        Producto other = (Producto) object;
        if ((this.id == null && other.id != null) || (this.id != null && !this.id.equals(other.id))) {
            return false;
        }
        return true;
    }
}