package BBUVA;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.Calendar;
import java.util.Date;
import java.util.Observable;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Sucursal extends Observable {

    private List<Producto> mProducto;
    private List<Remesa> mRemesa;
    private List<Empleado> mEmpleado;
    private List<Titular> mTitular;
    private List<Caja> mCaja;
    private int nSucursal;
    private ControladorBD cbd = new ControladorBD();

    public Sucursal () {
        mProducto = new ArrayList<Producto>();
        mRemesa = new ArrayList<Remesa>();
        mTitular = new ArrayList<Titular>();
        mEmpleado = new ArrayList<Empleado>();
        mCaja = new ArrayList<Caja>();
    }

    public void addProducto(Producto val) {
        mProducto.add(val);
        this.setChanged();
        this.notifyObservers();
    }

    public void addTitular(Titular val) {
        mTitular.add(val);
        this.setChanged();
        this.notifyObservers();
    }

    public void addTitularAProducto(String titular,String cuenta) {
        this.getTitularID(titular).addProducto(this.getProducto(cuenta));
        this.getProducto(cuenta).addTitular(this.getTitularID(titular));
        cbd.actualizar(this.getTitularID(titular));
    }

    public boolean anadirMovimientoCaja(float saldo, int numCaja, String descripcion) {
        if(this.getCaja(numCaja).getArq().getSaldoFinalTeorico()+saldo<0) {
            return false;
        }
        else {
            Movimiento m = new Movimiento();
            m.setCambioSaldo(saldo);
            m.setDescripcion(descripcion);
            Calendar actual= Calendar.getInstance();
            m.setFechaOperacion(actual.getTime());
            this.getCaja(numCaja).getArq().anadirMovimiento(m);
            this.setChanged();
            this.notifyObservers();
            return true;
        }
    }

    public String buscaNumeroNuevo() {
        List<String> lista=new ArrayList();
        for(int i=0;i<mProducto.size();i++)
            lista.add(mProducto.get(i).getNumeroProducto());

        String numero="2098"+ this.getNSucursal();
        String aux="";

        for(long cont=0;cont<1000000000;cont++) {
            aux=Long.toString(cont);
            for(int i=aux.length();i<12;i++) aux="0"+aux;
            if(!lista.contains(numero+aux))break;
        }
        numero+=aux;
        return numero;
    }

    public boolean empleadoCorrecto(int numero,String clave) {
        for(int i=0;i<this.getMEmpleado().size();i++)
            if(this.getMEmpleado().get(i).getNumOperario()==numero && this.getMEmpleado().get(i).getClave().compareTo(clave)==0)
                return true;
        return false;
    }

    public int empleadoDirector(int numero, String clave) {
        if(this.empleadoCorrecto(numero,clave)) {
            if(this.getEmpleado(numero).isDirector()) {
                return 1;
            }
            return 0;
        }
        return -1;
    }

    public String enviarParteDiferencia(int numEmp, int numCaja, float saldoReal) throws IOException {

        String path = "C:\\BBUVA";
        Calendar ca = Calendar.getInstance();
        path+="\\Parte de diferencia ";
        path+=ca.getTime().toString().substring(4,10)+" ";
        path+=ca.getTime().toString().substring(24,28);
        File fichero = new File(path);
        int i = 0;
        while(fichero.exists()) {
            i++;
            path = path.substring(0,40) + " " + i;
            fichero = new File(path);
        }
        fichero.createNewFile();
        FileWriter fw = new FileWriter(path);
        BufferedWriter bw = new BufferedWriter(fw);

        String parte="";
        String linea="";
        parte+=linea="Parte de diferencia:\n";
        bw.write(linea);
        bw.newLine();
        parte+=linea=" Suscursal: " + this.getNSucursal() + "  Caja :" + numCaja +"\n";
        bw.write(linea);
        bw.newLine();
        parte+=linea=" Saldo teórico: " + this.getCaja(numCaja).getArq().getSaldoFinalTeorico()+"\n";
        bw.write(linea);
        bw.newLine();
        parte+=linea=" Saldo real: " + saldoReal+"\n";
        bw.write(linea);
        bw.newLine();
        parte+=linea=" Operario: " + this.getEmpleado(numEmp).getNumOperario() + " - "+this.getEmpleado(numEmp).getNombre()+"\n";
        bw.write(linea);
        bw.newLine();
        Calendar c = Calendar.getInstance();
        parte+=linea=" "+c.getTime();
        bw.write(linea);
        bw.newLine();

        bw.close();
        fw.close();

        this.getCaja(numCaja).getArq().setSaldoFinal(saldoReal);
        this.finalizarArqueo(numEmp, numCaja,false);

        return parte;
    }

    public Empleado getEmpleado(int numero) {
        for(int i=0;i<this.mEmpleado.size();i++)
            if(this.mEmpleado.get(i).getNumOperario()==numero)
                return this.mEmpleado.get(i);
        return null;
    }

    public Caja getCaja(int numero) {
        for(int i=0;i<this.mCaja.size();i++)
            if(this.mCaja.get(i).getIdCaja()==numero)
                return this.mCaja.get(i);
        return null;
    }

    public List<Producto> getProducto () {
        return mProducto;
    }

    public void incorporaRemesaAbono(String path) throws ParseException, FileNotFoundException, IOException {
        FileReader fr = null;
        fr = new FileReader(path);
        BufferedReader bf = new BufferedReader(fr);
        String linea;
        String fechaRemesa="";
        String descripcion="";
        String numCta="";
        String numCtaOrd="";
        Remesa r=new Remesa();
        while ((linea = bf.readLine()) != null) {
            String buffer;
            buffer = linea.substring(0, 4);

            if(buffer.compareTo("0356")==0){
                if(linea.substring(26,29).compareTo("001")==0) {
                    fechaRemesa = linea.substring(29,35);
                    numCtaOrd = linea.substring(41,61);
                }
                if(linea.substring(26,29).compareTo("002")==0) {
                    r.setOrdenante(this.getTitularNom(linea.substring(29).trim()));
                }
            }else if(buffer.compareTo("0656")==0){
                if(linea.substring(26,29).compareTo("010")==0) {
                    OperRemesa op=new OperRemesa();
                    String valor=linea.substring(29,41);
                    while(valor.toCharArray()[0]=='0')
                        valor=valor.substring(1);
                    float n = Float.parseFloat(valor);
                    n /= 100;
                    op.setCambioSaldo(n);
                    descripcion = "Abono de " + r.getOrdenante().getNombre();
                    op.setDescripcion(descripcion);
                    SimpleDateFormat df2=new SimpleDateFormat("ddMMyy");
                    op.setFechaOperacion(df2.parse(fechaRemesa));
                    r.addOperRemesa(op);
                    numCta=linea.substring(41,61);
                    this.getProducto(numCta).getMovimiento().add(op);
                }
            }else if(buffer.compareTo("0856")==0) {
                Movimiento mov=new Movimiento();
                String valor=linea.substring(29,41);
                while(valor.toCharArray()[0]=='0')
                    valor=valor.substring(1);
                float n = Float.parseFloat(valor);
                n /= 100;
                mov.setCambioSaldo(n-1);
                SimpleDateFormat df2=new SimpleDateFormat("ddMMyy");
                mov.setFechaOperacion(df2.parse(fechaRemesa));
                mov.setDescripcion("Remesa " + df2.parse(fechaRemesa).toString());
                this.getProducto(numCtaOrd).getMovimiento().add(mov);
            }
        }
        this.mRemesa.add(r);
        this.setChanged();
        this.notifyObservers();
    }

    public boolean ingreso(String nCuenta,float importe,int caja) throws ParseException {
        Calendar actual= Calendar.getInstance();
        boolean retornar= this.anadirMovimiento(nCuenta,importe, "ingreso", actual.getTime());
        if(retornar)
            this.getCaja(caja).getArq().anadirMovimiento(this.getProducto(nCuenta).getMovimiento().get(this.getProducto(nCuenta).getMovimiento().size()-1));
        this.setChanged();
        this.notifyObservers();
        return retornar;
    }

    public void nuevaCuentaCorriente(String nID) {
        Producto nuevo = new CuentaCorriente();
        nuevo.setNumeroProducto(this.buscaNumeroNuevo());
        int i=this.posTitularID(nID);
        nuevo.addTitular(this.getTitular().get(i));
        this.getTitular().get(i).addProducto(nuevo);
        this.addProducto(nuevo);
        cbd.actualizar(this.getTitular().get(i));
    }

    public void nuevaLibretaAhorro(String nID) {
        Producto nuevo = new LibretaAhorro();
        nuevo.setNumeroProducto(this.buscaNumeroNuevo());
        int i=this.posTitularID(nID);
        nuevo.addTitular(this.getTitular().get(i));
        this.getTitular().get(i).addProducto(nuevo);
        this.addProducto(nuevo);
        cbd.actualizar(this.getTitular().get(i));
    }

    public boolean nuevaPersona(String nID, String nombre) {
        if(nID.trim().compareTo("")==0 || nombre.trim().compareTo("")==0) return false;
        else {
            Persona p=new Persona();
            p.setDocumentacion(nID);
            p.setNombre(nombre);
            this.addTitular(p);
            cbd.persist(p);
            return true;
        }
    }

    public boolean nuevaEmpresa(String nID, String nombre) {
        if(nID.trim().compareTo("")==0 || nombre.trim().compareTo("")==0) return false;
        else {
            Empresa e=new Empresa();
            e.setDocumentacion(nID);
            e.setNombre(nombre);
            this.addTitular(e);
            cbd.persist(e);
            return true;
        }
    }

    public void nuevoArqueo(int numEmp, int numCaja) {
        float saldo = this.getCaja(numCaja).getArq().getSaldoFinal();
        Arqueo ar = new Arqueo(saldo);
        this.getCaja(numCaja).setArq(ar);
        this.getCaja(numCaja).setLibre(false);
        this.getEmpleado(numEmp).setArq(ar);
        this.setChanged();
        this.notifyObservers();
    }

    public void finalizarArqueo(int numEmp, int numCaja, boolean actualizar) {
        if(actualizar) this.getCaja(numCaja).getArq().actualizarSaldoFinalComoTeorico();
        this.getCaja(numCaja).getArq().setFinalizado(true);
        this.getCaja(numCaja).setLibre(true);
        this.getEmpleado(numEmp).setArq(null);
    }

    public boolean prestamo(String nCuenta, float importe) {
        Calendar actual= Calendar.getInstance();
        boolean retornar= this.anadirMovimiento(nCuenta,importe, "Prestamo personal", actual.getTime());
        this.setChanged();
        this.notifyObservers();
        return retornar;
    }

    public boolean reintegro(String nCuenta, float importe,int caja) {
        Calendar actual= Calendar.getInstance();
        boolean retornar = this.anadirMovimiento(nCuenta,importe*-1, "reintegro", actual.getTime());
        if(retornar)
            this.getCaja(caja).getArq().anadirMovimiento(this.getProducto(nCuenta).getMovimiento().get(this.getProducto(nCuenta).getMovimiento().size()-1));
        this.setChanged();
        this.notifyObservers();
        return retornar;
    }

    public void setProducto (List<Producto> val) {
        this.mProducto = val;
    }

    public List<Remesa> getRemesa () {
        return mRemesa;
    }

    public void setRemesa (List<Remesa> val) {
        this.mRemesa = val;
    }

    public List<Titular> getTitular () {
        return mTitular;
    }

    public void setTitular (List<Titular> val) {
        this.mTitular = val;
    }

    public boolean transferenciaInterna(String origen, String destino, String importe) {
        Calendar c=Calendar.getInstance();
        if(origen.compareTo(destino)!=0)
            if(this.anadirMovimiento(origen, Float.parseFloat(importe)*-1, "Transferencia interna", c.getTime()) == true) {
                this.anadirMovimiento(destino, Float.parseFloat(importe), "Transferencia interna", c.getTime());
                return true;
            }
        return false;
    }

    public boolean transferenciaExterna(String origen, String destino, float importe) throws IOException, ParseException {
        Calendar c=Calendar.getInstance();
        float comision=1;
        if(origen.compareTo(destino)!=0)
            if(this.anadirMovimiento(origen,(((importe)*-1)+comision), "Transferencia interna", c.getTime()) == true) {
                this.envioCECA(destino,importe,"Transferencia");
                return true;
            }
        return false;
    }

    public String verRemesa (String path) {
        String resumenRemesa = "";
        try
        {
            FileReader fr = null;
            fr = new FileReader(path);
            BufferedReader bf = new BufferedReader(fr);
            String linea;
            while ((linea = bf.readLine()) != null) {
                if (linea.substring(0 ,4).compareTo("5180")==0) { //leo presentador
                    resumenRemesa+=linea.substring(28, 68)+"\n";
                }
                else if(linea.substring(0, 4).compareTo("5680")==0) {
                      resumenRemesa+=linea.substring(28, 68)+"\t"+linea.substring(88, 98)+"\n";
                }
           }
        }
        catch (IOException ex)
        {
            Logger.getLogger(Sucursal.class.getName()).log(Level.SEVERE, null, ex);
        }
        return resumenRemesa;
    }

    public void incorporaRemesa (String path) throws ParseException {
        try {
            FileReader fr = null;
            fr = new FileReader(path);
            BufferedReader bf = new BufferedReader(fr);
            String linea;
            String fechaRemesa="";
            String descripcion="";
            String numCta="";
            String numCtaOrd="";
            Remesa r=new Remesa();
            while ((linea = bf.readLine()) != null) {
                String buffer;
                buffer = linea.substring(0, 4);

                if(buffer.compareTo("5180")==0){
                    //registro cabecera presentador
                    r.setOrdenante(this.getTitularNom(linea.substring(28,68).trim()));
                }else if(buffer.compareTo("5380")==0){
                    fechaRemesa=linea.substring(22, 28);
                    numCtaOrd = linea.substring(68,88);
                }else if(buffer.compareTo("5680")==0){
                    OperRemesa op=new OperRemesa();
                    DecimalFormat df1=new DecimalFormat("#######,##");
                    Number n=df1.parse(linea.substring(88, 98));
                    op.setCambioSaldo(n.floatValue()*-1);
                    descripcion = linea.substring(114).trim();
                    op.setDescripcion(descripcion);
                    SimpleDateFormat df2=new SimpleDateFormat("ddMMyy");
                    op.setFechaOperacion(df2.parse(fechaRemesa));
                    r.addOperRemesa(op);
                    //leo el numcta
                    numCta=linea.substring(68,88);
                    this.getProducto(numCta).getMovimiento().add(op);
                } else if(linea.substring(0,4).compareTo("5681")==0) {
                    descripcion += " " + linea.substring(28,67).trim() + "\n" + linea.substring(68).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5682")==0) {
                    descripcion += "\n" + linea.substring(28,79).trim() + "\n" + linea.substring(108).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5683")==0) {
                    descripcion += " " + linea.substring(28,68).trim() + "\n" + linea.substring(68).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5684")==0) {
                    descripcion += "\n" + linea.substring(28,79) + "\n" + linea.substring(108).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5685")==0) {
                    descripcion += " " + linea.substring(28,39) + "\n" + linea.substring(68).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                } else if(linea.substring(0,4).compareTo("5686")==0) {
                    descripcion += "\n" + linea.substring(28).trim();
                    this.getProducto(numCta).getMovimiento().get(this.getProducto(numCta).getMovimiento().size()-1).setDescripcion(descripcion);
                }else if(buffer.compareTo("5880")==0) {
                    Movimiento mov=new Movimiento();
                    DecimalFormat df1=new DecimalFormat("#######,##");
                    Number n=df1.parse(linea.substring(88, 98));
                    mov.setCambioSaldo(n.floatValue());
                    SimpleDateFormat df2=new SimpleDateFormat("ddMMyy");
                    mov.setFechaOperacion(df2.parse(fechaRemesa));
                    mov.setDescripcion("Remesa " + df2.parse(fechaRemesa).toString());
                    this.getProducto(numCtaOrd).getMovimiento().add(mov);
                }
            }
            this.mRemesa.add(r);
            this.setChanged();
            this.notifyObservers();
        } catch (IOException ex) {
            Logger.getLogger(Sucursal.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    public Producto getProducto (String nCuenta) {
        for(int i=0;i<mProducto.size();i++)
            if(mProducto.get(i).getNumeroProducto().compareTo(nCuenta)==0)
                return mProducto.get(i);
        return null;
    }

    public int posTitularID (String ident) {
        for(int i=0;i<mTitular.size();i++)
            if(mTitular.get(i).getDocumentacion().compareTo(ident)==0)
                return i;
        return -1;
    }

    public int posTitularNombre (String nombre) {
        for(int i=0;i<mTitular.size();i++)
            if(mTitular.get(i).getNombre().compareTo(nombre)==0)
                return i;
        return -1;
    }
 
    public boolean anadirMovimiento (String nCuenta, float importe, String descrip, Date fecha) {
        if(this.getProducto(nCuenta).getSaldo()+importe<0 && importe<0) {
            return false; //da error
        }
        else { //TRATAMIENTO DE ERROR. Sólo entra si control (de error) es "true"
            Movimiento nuevo=new Movimiento();
            nuevo.setCambioSaldo(importe);
            nuevo.setDescripcion(descrip);
            nuevo.setFechaOperacion(fecha);
            this.getProducto(nCuenta).getMovimiento().add(nuevo);
            this.setChanged();
            this.notifyObservers();
            return true;
        }
    }

    public String verRemesaAbono(String path) throws FileNotFoundException, IOException {
        String resumenRemesa = "";
        FileReader fr = null;
        fr = new FileReader(path);
        BufferedReader bf = new BufferedReader(fr);
        String linea;
        while ((linea = bf.readLine()) != null) {
            if (linea.substring(0, 4).compareTo("0356") == 0) {
                //leo presentador
                if (linea.substring(26, 29).compareTo("002") == 0) {
                    resumenRemesa += linea.substring(29).trim() + "\n";
                }
            } else if (linea.substring(0, 4).compareTo("0656") == 0 || linea.substring(0, 4).compareTo("0657") == 0 || linea.substring(0, 4).compareTo("0658") == 0 || linea.substring(0, 4).compareTo("0659") == 0) {
                if (linea.substring(26, 29).compareTo("010") == 0) {
                    String valor=linea.substring(29,39);
                    while(valor.toCharArray()[0]=='0')
                        valor=valor.substring(1);
                    resumenRemesa += valor/*linea.substring(29, 39)*/ + "," + linea.substring(39, 41) + " €      ";
                } else if (linea.substring(26, 29).compareTo("011") == 0) {
                    resumenRemesa += linea.substring(29).trim() + "\n";
                }
            }
        }
        return resumenRemesa;
    }

    private void envioCECA(String destino, float importe, String descripcion) throws IOException, ParseException {
        String path = "C:\\BBUVA";
        Calendar c = Calendar.getInstance();
        path+="\\transferencias ";
        path+=c.getTime().toString().substring(4,10)+" ";
        path+=c.getTime().toString().substring(24,28);
        File fichero = new File(path);
        if(!fichero.exists()){
            fichero.createNewFile();
        }
        String texto="";
        long importeL = (long) (importe * 100);
        String importeS = Long.toString(importeL);
        while(importeS.length()<10) importeS = 0 + importeS;
        texto += "*" + destino + "  " + importeS + "  " + descripcion;

        FileWriter fw = new FileWriter(path,true);
        fw.write(texto);
        fw.close();
    }

    public List<Empleado> getMEmpleado() {
        return mEmpleado;
    }

    public void setMEmpleado(List<Empleado> mEmpleado) {
        this.mEmpleado = mEmpleado;
    }

    public List<Caja> getMCaja() {
        return mCaja;
    }

    public void setMCaja(List<Caja> mCaja) {
        this.mCaja = mCaja;
    }

    public Titular getTitularNom(String nombre) {
        for(int i=0;i<mTitular.size();i++)
            if(mTitular.get(i).getNombre().compareTo(nombre)==0)
                return mTitular.get(i);
        return null;
    }

    public Titular getTitularID(String identificacion) {
        for(int i=0;i<mTitular.size();i++)
            if(mTitular.get(i).getDocumentacion().compareTo(identificacion)==0)
                return mTitular.get(i);
        return null;
    }

    public int getNSucursal() {
        return nSucursal;
    }

    public void setNSucursal(int nSucursal) {
        this.nSucursal = nSucursal;
    }
}